/***************************************************************************
 *            socketClass.h
 *
 *  Thu Jul  1 22:21:48 2004
 *  Copyright  2004  Michael Rasmussen
 *  mir@miras.org
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef _SOCKETCLASS_H
#define _SOCKETCLASS_H

#include <iostream>
#include <string>
using namespace std;

#include "socket.h"

#define MAXBUFSIZE	4096

class SocketException {
	public:
		SocketException() : message ("Exception: Could not create socket") {};
		const char *exception() const { return message; };
	private:
		const char *message;
};

class ClientStub {
	public:
		ClientStub() : ip ("?.?.?.?"), host ("") {};
		string 		getIp() { return ip; };
		string		getHostname() { return host; };
		~ClientStub() {};

	protected:
		void		setClientData(const char *HOST, const char *IP) { host=HOST; ip=IP; };

	private:
		string		ip;
		string		host;
};

class SocketServer : public ClientStub {
	public:
		SocketServer() {};
		/* SocketServer(<port>, <backlog>)                       */
		SocketServer(int, int);
		~SocketServer() { Stop(); };
		bool 		Start();
		/* Read(<buffer>, <max characters to read>)              */
		/* Stop condition: reads until max or input = '\0'       */
		int			Read(string &, int);
		/* Readln(<buffer>)                                      */
		/* Stop condition: read until MAXBUFSIZE or input = '\n' */
		int			Readln(string &);
		int			Write(string &);
		int			Write(const char *);
		void		Init(int, int);
		void		Close();
		string		getClientIp();
		string		getClientHost();
		/* These methods are vital if the class is used in a threaded environment */
		void		setFd(int);
		int			getFd();
		
	private:
		void 		Stop();
		Tcpsocket	server;
		int			fd;

};

#endif /* _SOCKETCLASS_H */
