/***************************************************************************
 *            main.cc
 *
 *  Thu Jul  1 20:48:52 2004
 *  Copyright  2004  Michael Rasmussen
 *  mir@miras.org
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 

#include <stdio.h>
#include <signal.h>
#include <pthread.h>
#include <unistd.h>
#include <sys/types.h>
#include "socketClass.h"

int			fport = 1234;
int			tport = 1235;
int			maxcon = 0;

// Function prototypes
int forkWorker();
void setup(pthread_attr_t *);
extern "C" void *handleThreadChild(void *);
int threadWorker();

int main()
{
pid_t	pid;
	
	pid = fork();
	if (pid == 0) {
		if (forkWorker())
			cerr << "Unable to run a forked socket" << endl;
		exit(EXIT_SUCCESS);
	}
	if (threadWorker())
		cerr << "Unable to run a threaded socket" << endl;
	exit(EXIT_SUCCESS);
}

int forkWorker() {	
SocketServer *server;
string       buffer;
pid_t        cid;
	
	signal(SIGCHLD, noZombie);
	cerr << "A forked version" << endl;
	try {
		server = new SocketServer(fport, maxcon);
	} catch (SocketException se) {
		cerr << se.exception() << endl;
		exit(EXIT_FAILURE);
	}
	while (1) {
		if (!server->Start()) {
			cerr << "Unable to run a forked socket" << endl;
			exit(EXIT_FAILURE);
		}
		cerr << "I got a call from: " << endl <<
		"IP: " << server->getClientIp() << " " <<
		"Host: " << server->getHostname() << endl;
		cid = fork();
		if (cid == 0) {
			cerr << "In fork child" << endl;
			while (1) {
				server->Write("input: ");
				if (server->Readln(buffer) < 0)
					cerr << "Error: recieving" << endl;
				cerr << server->getClientIp() << "->" << buffer << endl;
				if (buffer.compare("") == 0)
					break;
				buffer = "Recieved: " + buffer + "\n";
				server->Write(buffer);
			}
			server->Close();
			cerr << "Finish reading in forked child" << endl;
			exit(EXIT_SUCCESS);
		}
		server->Close();
	}
	return EXIT_SUCCESS;
}

void setup(pthread_attr_t *attrp) {
	pthread_attr_init(attrp);
	pthread_attr_setdetachstate(attrp, PTHREAD_CREATE_DETACHED);
}

extern "C" void *handleThreadChild(void *_s) {
int             fd;
SocketServer    *sock;
string          buffer;

	try {
		sock = new SocketServer();
	} catch (SocketException se) {
		cerr << se.exception() << endl;
		exit(EXIT_FAILURE);
	}
	fd = *(int *) _s;
	cerr << "In thread child." << endl;
	sock->setFd(fd);
	while(1) {
		sock->Write("input: ");
		if (sock->Readln(buffer) < 0)
			cerr << "Error: recieving" << endl;
		cerr << sock->getClientIp() << "->" << buffer << endl;
		if (buffer.compare("") == 0)
			break;
		buffer = "Recieved: " + buffer + "\n";
		sock->Write(buffer);
		buffer = "";
	}
	cerr << "Finish reading in threaded child. fd" << endl;
	sock->Close();
	return (void *) EXIT_SUCCESS;
}

int threadWorker() {	
pthread_t       worker;
pthread_attr_t  attr;
SocketServer    server;
int             fd;
	
	setup(&attr);
	cerr << "A threaded version" << endl;
	try {
		server.Init(tport, maxcon);
	} catch (SocketException se) {
		cerr << se.exception() << endl;
		exit(EXIT_FAILURE);
	}
	while (1) {
		if (!server.Start()) {
			cerr << "Unable to run a threaded socket" << endl;
			exit(EXIT_FAILURE);
		}
		fd = server.getFd();
		cerr << "ThreadChild: I got a call from: " << endl <<
		"IP: " << server.getClientIp() << " " <<
		"Host: " << server.getHostname() << endl;
		pthread_create(&worker, &attr, handleThreadChild, (void *) &fd);
	}
	return EXIT_SUCCESS;
}
